@@ -53,6 +53,7 @@ _ACTIVATION_FLAG_EXISTING_ONLY = 1<<2; # don't start the server if not started
 song_change_prefs = gconfsync.GConfSync("/apps/gnome-osd/plugins/song_change")
 xchat_prefs = gconfsync.GConfSync("/apps/gnome-osd/plugins/xchat")
 email_prefs = gconfsync.GConfSync("/apps/gnome-osd/plugins/email")
+gaim_prefs = gconfsync.GConfSync("/apps/gnome-osd/plugins/gaim")
 listener = None # Bonobo/ObjectDirectory:activation:register listener
 screensaver_active = False
 
@@ -222,12 +223,16 @@ def die_cb(cli):
     bonobo.main_quit()
     print "OK, quitting..."
 
-def evolution_new_mail_callback(arg):
+def evolution_new_mail_callback(uri, folder=None):
     global email_prefs, osd
     if not email_prefs['enabled']:
         return
-    if not arg.upper().endswith("INBOX"):
-        return
+    if folder is None:
+        if not uri.upper().endswith("INBOX"):
+            return
+    else:
+        if folder.upper() != "INBOX":
+            return
     osd.send("<message id='newmail' hide_timeout='3000'>You Have Mail</message>")
 
 def muine_song_change_cb(arg):
@@ -253,9 +258,23 @@ XCHAT_EAT_XCHAT = 1        # don't let xchat see this event
 XCHAT_EAT_PLUGIN = 2       # don't let other plugins see this event
 XCHAT_EAT_ALL =	(XCHAT_EAT_XCHAT|XCHAT_EAT_PLUGIN) # don't let anything see this event
 
+try:
+    _xchat_ver = [int(x) for x in os.popen("xchat --version 2> /dev/null").read().split()[1].split('.')]
+except IndexError:
+    _xchat_ver = None
+
+XCHAT_DBUS_SERVICE = "org.xchat.service"
+if _xchat_ver >= [2,8]:
+    XCHAT_DBUS_OBJECT = "/org/xchat/Remote"
+    XCHAT_DBUS_INTERFACE = "org.xchat.plugin"
+else:
+    XCHAT_DBUS_OBJECT = "/org/xchat/RemoteObject"
+    XCHAT_DBUS_INTERFACE = "org.xchat.interface"
+del _xchat_ver
+
 def monitor_xchat(bus):
-    remote_object = bus.get_object("org.xchat.service", "/org/xchat/RemoteObject")
-    xchat = dbus.Interface(remote_object, "org.xchat.interface")
+    remote_object = bus.get_object(XCHAT_DBUS_SERVICE, XCHAT_DBUS_OBJECT)
+    xchat = dbus.Interface(remote_object, XCHAT_DBUS_INTERFACE)
     event_ids = []
     for event in ('Channel Action Hilight', 'Channel Msg Hilight',
                   'Private Message', 'Private Message to Dialog'):
@@ -263,7 +282,7 @@ def monitor_xchat(bus):
         event_ids.append(id_)
         atexit.register(xchat.unhook, id_)
         
-    def print_cb(word, id_):
+    def print_cb(word, id_, context_id=None):
         global xchat_prefs, osd
         if id_ not in event_ids:
             return
@@ -282,12 +301,14 @@ def monitor_xchat(bus):
         osd.send(message)
     
     remote_object.connect_to_signal("PrintSignal", print_cb,
-                                    dbus_interface="org.xchat.interface")
-    
+                                    dbus_interface=XCHAT_DBUS_INTERFACE)
 
 
 def xchat_registered_cb(*args):
-    monitor_xchat(dbus.SessionBus())
+    try:
+        monitor_xchat(dbus.SessionBus())
+    except dbus.DBusException, ex:
+        print "XChat monitoring is not working:", ex
 
 def screensaver_active_cb(active):
     global screensaver_active
@@ -316,7 +337,7 @@ def rhythmbox_dbus_song_change_cb(uri):
         return
     bus = dbus.SessionBus()
     rbshellobj = bus.get_object('org.gnome.Rhythmbox', '/org/gnome/Rhythmbox/Shell')
-    rbshell = rbshell = dbus.Interface(rbshellobj, 'org.gnome.Rhythmbox.Shell')
+    rbshell = dbus.Interface(rbshellobj, 'org.gnome.Rhythmbox.Shell')
     props = rbshell.getSongProperties(uri)
     info = SongInfo()
     info.title = props['title']
@@ -328,6 +349,73 @@ def rhythmbox_dbus_song_change_cb(uri):
     osd.send("<message id='rhythmbox' hide_timeout='10000'>" + song + "</message>")
 
 
+def format_pgaim(buddy, action):
+    return "<span foreground='yellow'>%s</span> " \
+           "<span size='smaller' style='italic'>%s</span>" % (buddy, action)
+
+def gaim_event_buddyon_cb(buddyid):
+    global gaim_prefs, osd
+    if not gaim_prefs['enabled']:
+        return
+    bus = dbus.SessionBus()
+    gaimobj = bus.get_object("net.sf.gaim.GaimService", "/net/sf/gaim/GaimObject")
+    gaim = dbus.Interface(gaimobj, "net.sf.gaim.GaimInterface")
+    alias = gaim.GaimBuddyGetAlias(buddyid)
+    message = format_pgaim(alias, "logs in")
+    osd.send("<message id='gaim' hide_timeout='3000'>" + message + "</message>")
+
+def gaim_event_buddyoff_cb(buddyid):
+    global gaim_prefs, osd
+    if not gaim_prefs['enabled']:
+        return
+    bus = dbus.SessionBus()
+    gaimobj = bus.get_object("net.sf.gaim.GaimService", "/net/sf/gaim/GaimObject")
+    gaim = dbus.Interface(gaimobj, "net.sf.gaim.GaimInterface")
+    alias = gaim.GaimBuddyGetAlias(buddyid)
+    message = format_pgaim(alias, "logs out")
+    osd.send("<message id='gaim' hide_timeout='3000'>" + message + "</message>")
+
+def gaim_event_message_cb(account, name):
+    global gaim_prefs, osd
+    if not gaim_prefs['enabled']:
+        return
+    bus = dbus.SessionBus()
+    gaimobj = bus.get_object("net.sf.gaim.GaimService", "/net/sf/gaim/GaimObject")
+    gaim = dbus.Interface(gaimobj, "net.sf.gaim.GaimInterface")
+    buddy = gaim.GaimFindBuddy(account, name)
+    if buddy != 0:
+        alias = gaim.GaimBuddyGetAlias(buddy)
+    else:
+        alias = name
+    message = format_pgaim(alias, "is writting a Message")
+    osd.send("<message id='gaim' hide_timeout='3000'>" + message + "</message>")
+
+def gaim_event_messagereceived_cb(account, name, message, conversation, flags):
+    global gaim_prefs, osd
+    if not gaim_prefs['enabled']:
+        return
+    bus = dbus.SessionBus()
+    gaimobj = bus.get_object("net.sf.gaim.GaimService", "/net/sf/gaim/GaimObject")
+    gaim = dbus.Interface(gaimobj, "net.sf.gaim.GaimInterface")
+    buddy = gaim.GaimFindBuddy(account, name)
+    if buddy != 0:
+        alias = gaim.GaimBuddyGetAlias(buddy)
+    else:
+        alias = name
+    message = format_pgaim(alias, "sent a Message")
+    osd.send("<message id='gaim' hide_timeout='3000'>" + message + "</message>")
+
+def monitor_gaim(bus):
+    bus.add_signal_receiver(gaim_event_buddyon_cb, "BuddySignedOn",
+                            "net.sf.gaim.GaimInterface")
+    bus.add_signal_receiver(gaim_event_buddyoff_cb, "BuddySignedOff",
+                            "net.sf.gaim.GaimInterface")
+    bus.add_signal_receiver(gaim_event_message_cb, "BuddyTyping",
+                            "net.sf.gaim.GaimInterface")
+    bus.add_signal_receiver(gaim_event_messagereceived_cb, "ReceivedImMsg",
+                            "net.sf.gaim.GaimInterface")
+
+
 def monitor_dbus_apps():
     bus = dbus.SessionBus()
     bus.add_signal_receiver(evolution_new_mail_callback, "Newmail",
@@ -337,27 +425,40 @@ def monitor_dbus_apps():
     bus.add_signal_receiver(rhythmbox_dbus_song_change_cb, "playingUriChanged",
                             "org.gnome.Rhythmbox.Player")
     bus.add_signal_receiver(xchat_registered_cb, "NameOwnerChanged",
-                            "org.freedesktop.DBus", arg0="org.xchat.service")
+                            "org.freedesktop.DBus", arg0=XCHAT_DBUS_SERVICE)
     bus.add_signal_receiver(screensaver_active_cb, "ActiveChanged",
                             "org.gnome.ScreenSaver")
+    monitor_gaim(bus)
     try:
         monitor_xchat(bus)
-    except dbus.DBusException:
-        pass
+    except dbus.DBusException, ex:
+        print "XChat monitoring is not working:", ex
 
 if HAVE_DBUS:
+    _our_bus_name = None
+
     def start_dbus_interface():
+        global _our_bus_name
         session_bus = dbus.SessionBus()
         bus = dbus.SessionBus()
-        retval = dbus.dbus_bindings.bus_request_name(session_bus.get_connection(),
-                                                     "pt.inescporto.telecom.GnomeOSD.EventBridge",
-                                                     dbus.dbus_bindings.NAME_FLAG_DO_NOT_QUEUE)
-        if retval in (dbus.dbus_bindings.REQUEST_NAME_REPLY_PRIMARY_OWNER,
-                      dbus.dbus_bindings.REQUEST_NAME_REPLY_ALREADY_OWNER):
-            pass
-        elif retval in (dbus.dbus_bindings.REQUEST_NAME_REPLY_EXISTS,
-                        dbus.dbus_bindings.REQUEST_NAME_REPLY_IN_QUEUE):
-            raise SystemExit("already running")
+        if dbus.version >= (0,80):
+            _our_bus_name = None
+            # need to keep the BusName referenced in order to hang on to the name
+            try:
+                _our_bus_name = dbus.service.BusName("pt.inescporto.telecom.GnomeOSD.EventBridge",
+                                                 bus, do_not_queue=True)
+            except dbus.NameExistsException:
+                raise SystemExit("already running")
+        else:
+            retval = dbus.dbus_bindings.bus_request_name(session_bus.get_connection(),
+                                                         "pt.inescporto.telecom.GnomeOSD.EventBridge",
+                                                         dbus.dbus_bindings.NAME_FLAG_DO_NOT_QUEUE)
+            if retval in (dbus.dbus_bindings.REQUEST_NAME_REPLY_PRIMARY_OWNER,
+                          dbus.dbus_bindings.REQUEST_NAME_REPLY_ALREADY_OWNER):
+                pass
+            elif retval in (dbus.dbus_bindings.REQUEST_NAME_REPLY_EXISTS,
+                            dbus.dbus_bindings.REQUEST_NAME_REPLY_IN_QUEUE):
+                raise SystemExit("already running")
 
 
 def main():
